/* LeMoNe - Learning Module Network
 * 
 * Copyright (C) 2005-2008 VIB 
 * 
 * Written by Tom Michoel, Eric Bonnet, Anagha Joshi, Steven Maere
 *  
*/

package be.ugent.psb.ModuleNetwork;

import java.util.*;

import org.apache.commons.cli.*;
import java.io.*;

/**
 * 
 * Run LeMoNe with command line options
 * 
 * @author erbon
 *
 */
public class RunCli {
	

	/**
	 * Parse command line options and run LeMoNe 
	 * 
	 * @param args command-line arguments string
	 *            
	 */
	public static void main(String[] args) {
		
		// set dummy values for those parameters, they'll be filled later
		String task = null;
		String data_dir = null;
		String data_file = null;
		String mir_file = null;
		String reg_file = null;
		String output_file = null;
		String range = null;
		int num_steps = 0;
		int burn_in = 0;
		int sample_steps = 0;
		String cluster_file = null;
		int num_clust = 0;
		String go_annot_file = null;
		String go_ref_file = null;
		String top_regulators = null;
		String map_file = null;
		
		// set default values for those parameters, users can override them
		double alpha = 0.1;
		double beta = 0.1;
		double mu = 0.0;
		double lambda = 0.1;
		double score_gain = 0;
		int init_num_clust = 0;
		int num_runs = 1;
		boolean use_bayesian_score = true;
		int num_reg = 10;
		double beta_reg = 20;
		boolean help = false;
		boolean mir = false;
		boolean matlab = false;
		String go_p_value = "0.05";
		String go_ontology_file = "GO_Biological_Process";
		boolean go_annot_def = true;
		boolean go_ontology_def = true;
		boolean use_global_mean = false;
		boolean use_regulator_mean = false;
		boolean all_regulators = false;
		int cut_level = 0;
		
		// create the different options
		Options opts = new Options();
		opts.addOption("task", true, "task to perform");
		opts.addOption("data_dir", true, "data directory");
		opts.addOption("data_file", true, "data file (genes)");
		opts.addOption("reg_file", true, "regulators file");
		opts.addOption("mir_file", true, "mir file");
		opts.addOption("output_file", true, "output file name");
		opts.addOption("num_steps", true, "number of steps (Gibbs sampler)");
		opts.addOption("burn_in", true, "number of burn-in steps (Gibbs sampler)");
		opts.addOption("sample_steps", true, "sample steps interval (Gibbs sampler)");
		opts.addOption("cluster_file", true, "cluster file name");
		opts.addOption("num_clust", true, "number of clusters");
		opts.addOption("alpha", true, "alpha0 parameter value");
		opts.addOption("beta", true, "beta0 parameter value");
		opts.addOption("mu", true, "mu0 parameter value");
		opts.addOption("lambda", true, "lambda0 parameter value");
		opts.addOption("score_gain", true, "score gain cutoff value");
		opts.addOption("init_num_clust", true, "initial number of clusters (Gibbs sampler)");
		opts.addOption("num_runs", true, "number of runs (Gibbs sampler)");
		opts.addOption("use_bayesian_score", true, "use bayesian score computation");
		opts.addOption("num_reg", true, "maximum number of regulators assigned for each node");
		opts.addOption("beta_reg", true, "beta parameter value for regulators assignment");
		opts.addOption("signi_fact", true, "multiplication factor in split significance test (consensus trees)");
		opts.addOption("num_split",true, "number of splits for the module set");
		opts.addOption("prefix",true, "java command prefix for the split option command line");
		opts.addOption("range",true, "module set range for the assignment of the regulators");
		opts.addOption("mir", false, "microRNA scheme (set of genes + extra regulators)");
		opts.addOption("go_annot_file", true, "GO custom annotation file");
		opts.addOption("go_ontology_file", true, "GO ontology file name");
		opts.addOption("go_ref_file", true, "GO refence gene list file name");
		opts.addOption("go_p_value", true, "GO p-value cutoff");
		opts.addOption("go_ontology_def", false, "GO ontology file flag");
		opts.addOption("go_annot_def", false, "GO annotation file flag");
		opts.addOption("matlab", false, "Matlab format for output files");
		opts.addOption("help", false, "help");
		opts.addOption("h", false, "help");
		opts.addOption("top_regulators", true, "Top regulators file name");
		opts.addOption("use_global_mean", false, "Use global mean for the figures");
		opts.addOption("use_regulator_mean", false, "Use regulator mean for the figures");
		opts.addOption("all_regulators", false, "Print all regulators");
		opts.addOption("map_file", true, "Gene names map file");
		opts.addOption("cut_level", true, "Regulation tree cut level");
		
		// build a parser object and parse the command line (!)
		CommandLineParser parser = new PosixParser();
		try {
			
			CommandLine cmd = parser.parse(opts, args);
			
			if (cmd.hasOption("task"))
				task = cmd.getOptionValue("task");
			if (cmd.hasOption("data_dir")) 
				data_dir = cmd.getOptionValue("data_dir");
			if (cmd.hasOption("data_file"))
				data_file = cmd.getOptionValue("data_file");
			if (cmd.hasOption("reg_file"))
				reg_file = cmd.getOptionValue("reg_file");
			if (cmd.hasOption("mir_file"))
				mir_file = cmd.getOptionValue("mir_file");
			if (cmd.hasOption("output_file"))
				output_file = cmd.getOptionValue("output_file");
			if (cmd.hasOption("cluster_file"))
				cluster_file = cmd.getOptionValue("cluster_file");
			if (cmd.hasOption("help") || cmd.hasOption("h"))
				help = true;
			if (cmd.hasOption("alpha"))
				alpha = Double.parseDouble(cmd.getOptionValue("alpha"));
			if (cmd.hasOption("beta"))
				beta = Double.parseDouble(cmd.getOptionValue("beta"));
			if (cmd.hasOption("lambda"))
				lambda = Double.parseDouble(cmd.getOptionValue("lambda"));
			if (cmd.hasOption("mu"))
				mu = Double.parseDouble(cmd.getOptionValue("mu"));
			if (cmd.hasOption("score_gain"))
				score_gain = Double.parseDouble(cmd.getOptionValue("score_gain"));
			if (cmd.hasOption("num_runs"))
				num_runs = Integer.parseInt(cmd.getOptionValue("num_runs"));
			if (cmd.hasOption("num_steps"))
				num_steps = Integer.parseInt(cmd.getOptionValue("num_steps"));
			if (cmd.hasOption("burn_in"))
				burn_in = Integer.parseInt(cmd.getOptionValue("burn_in"));
			if (cmd.hasOption("sample_steps"))
				sample_steps = Integer.parseInt(cmd.getOptionValue("sample_steps"));
			if (cmd.hasOption("init_num_clust"))
				init_num_clust = Integer.parseInt(cmd.getOptionValue("init_num_clust"));
			if (cmd.hasOption("use_bayesian_score"))
				use_bayesian_score = Boolean.parseBoolean(cmd.getOptionValue("use_bayesian_score"));
			if (cmd.hasOption("num_reg"))
				num_reg = Integer.parseInt(cmd.getOptionValue("num_reg"));
			if (cmd.hasOption("beta_reg"))
				beta_reg = Double.parseDouble(cmd.getOptionValue("beta_reg"));
			if (cmd.hasOption("num_clust"))
				num_clust = Integer.parseInt(cmd.getOptionValue("num_clust"));
			if (cmd.hasOption("range"))
				range = cmd.getOptionValue("range");
			if (cmd.hasOption("mir"))
				mir = true;
			if (cmd.hasOption("matlab"))
				matlab = true;
			if (cmd.hasOption("go_annot_file"))
				go_annot_file = cmd.getOptionValue("go_annot_file");
			if (cmd.hasOption("go_ontology_file"))
				go_ontology_file = cmd.getOptionValue("go_ontology_file");
			if (cmd.hasOption("go_ref_file"))
				go_ref_file = cmd.getOptionValue("go_ref_file");
			if (cmd.hasOption("go_p_value"))
				go_p_value = cmd.getOptionValue("go_p_value");
			if (cmd.hasOption("go_ontology_def"))
				go_ontology_def = false;
			if (cmd.hasOption("top_regulators"))
				top_regulators = cmd.getOptionValue("top_regulators");
			if (cmd.hasOption("use_global_mean"))
				use_global_mean = true;
			if (cmd.hasOption("use_regulator_mean"))
				use_regulator_mean = true;
			if (cmd.hasOption("all_regulators"))
				all_regulators = true;
			if (cmd.hasOption("map_file"))
				map_file = cmd.getOptionValue("map_file");
			if (cmd.hasOption("cut_level"))
				cut_level = Integer.parseInt(cmd.getOptionValue("cut_level"));
			
		}
		catch (ParseException exp) {
			System.out.println("Error while parsing command line.");
			exp.printStackTrace();
			System.exit(1);
		}
		
		// print header
		printBanner();
		
		// print a (very) basic help
		if (help == true) {
			HelpFormatter formatter = new HelpFormatter();
			formatter.printHelp( "LeMoNe", opts );
			System.exit(1);
		}
		
		// something has to be done, we need a task to be set
		if (task == null)
			Die("Error: task option must be set.");
		
		// ---------------------------------------------------------------
		// ganesh task: 2-way clustering of genes using the gibbs sampler
		// ---------------------------------------------------------------
		if (task.equalsIgnoreCase("ganesh")) {

			// those parameters must be set
			if (data_dir == null)
				Die("Error: data_dir option must be set.");
			if (data_file == null)
				Die("Error: data_file option must be set.");
			if (output_file == null) 
				Die("Error: output_file option must be set.");
			if (init_num_clust == 0)
				Die("Error: init_num_clust option must be set.");
			
			// set default values if the user did not change them
			if (num_runs == 0)
				num_runs = 1;
			if (num_steps == 0)
				num_steps = 100;
			if (burn_in == 0)
				burn_in = 50;
			if (sample_steps == 0)
				sample_steps = 100;

			System.out.println("Parameters");
			System.out.println("----------");
			System.out.println("task:               " + task);
			System.out.println("data_dir:           " + data_dir);
			System.out.println("data_file:          " + data_file);
			System.out.println("output_file:        " + output_file);
			System.out.println("lambda:             " + lambda);
			System.out.println("mu:                 " + mu);
			System.out.println("alpha:              " + alpha);
			System.out.println("beta:               " + beta);
			System.out.println("init_num_clust:     " + init_num_clust);
			System.out.println("num_runs:           " + num_runs);
			System.out.println("num_steps:          " + num_steps);
			System.out.println("burn_in:            " + burn_in);
			System.out.println("sample_steps:       " + sample_steps);
			System.out.println("score_gain:         " + score_gain);
			System.out.println("use_bayesian_score: " + use_bayesian_score);
			System.out.println("mir:                " + mir);
			
			HashMap<String, String> empty = new HashMap<String, String>();

			if (mir == true) {
				// Create ModuleNetwork "mir" object
				ModuleNetwork M = new ModuleNetwork(data_dir, data_file, mir_file, reg_file, lambda, mu, alpha, beta);
				// Gibbs sample different module sets with one tree per module
				M.gibbsSamplerGenes(init_num_clust, num_runs, num_steps, burn_in, sample_steps, score_gain, use_bayesian_score);
				// write results
				M.writeMicroXML(output_file + ".xml.gz");
			}
			else {
				// Create ModuleNetwork object
				ModuleNetwork M = new ModuleNetwork(data_dir, data_file, reg_file, lambda, mu, alpha, beta, empty);
				// Gibbs sample different module sets with one tree per module
				M.gibbsSamplerGenes(init_num_clust, num_runs, num_steps, burn_in, sample_steps, score_gain, use_bayesian_score);
				// write results
				M.writeXML(output_file + ".xml.gz");
			}
						 
		 }
		 //-------------------------------------------------------------------------------------
		 // regulators task: learn regulation programs (gibbs sampling exp. + assign regulators)
		 //-------------------------------------------------------------------------------------
		 else if (task.equalsIgnoreCase("regulators")) {
			
			// those parameters must be set
			if (data_dir == null)
				Die("Error: data_dir option must be set.");
			if (data_file == null)
				Die("Error: data_file option must be set.");
			if (mir == true && mir_file == null)
				Die("Error: mir_file option must be set.");
			if (reg_file == null)
				Die("Error: reg_file option must be set.");
			if (cluster_file == null)
				Die("Error: cluster_file option must be set.");
			if (output_file == null)
				Die("Error: output_file option must be set.");
			if (num_clust == 0)
				Die("Error: num_clust option must be set.");
			
			// set default values if the user did not change them
			if (num_runs == 0)
				num_runs = 1;
			if (num_steps == 0)
				num_steps = 1100;
			if (burn_in == 0)
				burn_in = 100;
			if (sample_steps == 0)
				sample_steps = 100;

			System.out.println("Parameters");
			System.out.println("----------");
			System.out.println("task:               " + task);
			System.out.println("data_dir:           " + data_dir);
			System.out.println("data_file:          " + data_file);
			if (mir == true)
				System.out.println("mir_file:           " + mir_file);
			System.out.println("reg_file:           " + reg_file);
			System.out.println("cluster_file:       " + cluster_file);
			System.out.println("num_clust:          " + num_clust);
			System.out.println("output_file:        " + output_file);
			System.out.println("lambda:             " + lambda);
			System.out.println("mu:                 " + mu);
			System.out.println("alpha:              " + alpha);
			System.out.println("beta:               " + beta);
			System.out.println("num_runs:           " + num_runs);
			System.out.println("num_steps:          " + num_steps);
			System.out.println("burn_in:            " + burn_in);
			System.out.println("sample_steps:       " + sample_steps);
			System.out.println("score_gain:         " + score_gain);
			System.out.println("num_reg:            " + num_reg);
			System.out.println("use_bayesian_score: " + use_bayesian_score);
			System.out.println("mir:                " + mir);
			
			HashMap<String,String> empty = new HashMap<String,String>();
			
			if (mir == true) {
				// create module network object
				ModuleNetwork M = new ModuleNetwork(data_dir, data_file, mir_file, num_clust, cluster_file, reg_file, lambda, mu, alpha, beta, empty);
				// default datatype
				M.dataType = M.dataType.RATIO;
				// cluster experiments using the gibbs sampler
				M.gibbsSamplerExpts(num_runs, num_steps, burn_in, sample_steps, score_gain, use_bayesian_score);
				// assign regulators
				M.assignRegulatorsNoAcyclStoch(beta_reg, num_reg);
				// write results as xml file
				M.writeMicroXML(output_file + ".xml.gz");
			}
			else {
				// create module network object
				ModuleNetwork M = new ModuleNetwork(data_dir, data_file, num_clust, cluster_file, reg_file, lambda, mu, alpha, beta, empty);
				// default datatype
				M.dataType = M.dataType.RATIO;
				// cluster experiments using the gibbs sampler
				M.gibbsSamplerExpts(num_runs, num_steps, burn_in, sample_steps, score_gain, use_bayesian_score);
				// assign regulators
				M.assignRegulatorsNoAcyclStoch(beta_reg, num_reg);
				// write results as xml file
				M.writeXML(output_file + ".xml.gz");
			}
			
		} 
		//----------------------------------------------------------
		// experiments task: cluster conditions using gibbs sampling 
		//----------------------------------------------------------
		 else if (task.equalsIgnoreCase("experiments")) {
			
			// those parameters must be set
			if (data_dir == null)
				Die("Error: data_dir option must be set.");
			if (data_file == null)
				Die("Error: data_file option must be set.");
			if (cluster_file == null)
				Die("Error: cluster_file option must be set.");
			if (output_file == null)
				Die("Error: output_file option must be set.");
			if (num_clust == 0)
				Die("Error: num_clust option must be set.");
			
			// set default values if the user did not change them
			if (num_runs == 0)
				num_runs = 1;
			if (num_steps == 0)
				num_steps = 1100;
			if (burn_in == 0)
				burn_in = 100;
			if (sample_steps == 0)
				sample_steps = 100;

			System.out.println("Parameters");
			System.out.println("----------");
			System.out.println("task:               " + task);
			System.out.println("data_dir:           " + data_dir);
			System.out.println("data_file:          " + data_file);
			System.out.println("reg_file:           " + reg_file);
			System.out.println("cluster_file:       " + cluster_file);
			System.out.println("num_clust:          " + num_clust);
			System.out.println("output_file:        " + output_file);
			System.out.println("lambda:             " + lambda);
			System.out.println("mu:                 " + mu);
			System.out.println("alpha:              " + alpha);
			System.out.println("beta:               " + beta);
			System.out.println("num_runs:           " + num_runs);
			System.out.println("num_steps:          " + num_steps);
			System.out.println("burn_in:            " + burn_in);
			System.out.println("sample_steps:       " + sample_steps);
			System.out.println("score_gain:         " + score_gain);
			System.out.println("use_bayesian_score: " + use_bayesian_score);
			
			HashMap<String,String> empty = new HashMap<String,String>();
			
			// create module network object
			ModuleNetwork M = new ModuleNetwork(data_dir, data_file, num_clust, cluster_file, reg_file, lambda, mu, alpha, beta, empty);
			
			// default datatype
			M.dataType = M.dataType.RATIO;

			// cluster experiments using the gibbs sampler
			M.gibbsSamplerExpts(num_runs, num_steps, burn_in, sample_steps, score_gain, use_bayesian_score);

			// write results as xml file
			M.writeXML(output_file + ".xml.gz");
		}
		//---------------------------------------------------------------
		// split_reg task: assign regulators for a given range of modules
		//---------------------------------------------------------------
		else if (task.equalsIgnoreCase("split_reg")) {
			
			// those parameters must be set
			if (data_dir == null)
				Die("Error: data_dir option must be set.");
			if (output_file == null)
				Die("Error: output_file option must be set.");
			
			System.out.println("Parameters");
			System.out.println("----------");
			System.out.println("task:               " + task);
			System.out.println("data_dir:           " + data_dir);
			System.out.println("output_file:        " + output_file);
			System.out.println("num_reg:            " + num_reg);
			System.out.println("beta_reg:           " + beta_reg);
			System.out.println("range:              " + range);
			
			// build module network object from every xml files in a directory
			ModuleNetwork M = new ModuleNetwork(data_dir,"",false);
				        
			String[] val = range.split(":");
			int start_module = Integer.parseInt(val[0]);
			int stop_module = Integer.parseInt(val[1]);
			
			// assign regulators
			M.assignRegulatorsNoAcyclStoch(beta_reg, num_reg, start_module, stop_module);
	        
			// write results as xml file
			M.writeXML(output_file + ".xml.gz");
		}
		//--------------------------------------------------------------------------------------------
		// split_concatenate task: concatenate several "split" files into one with all the information
		//--------------------------------------------------------------------------------------------
		else if (task.equalsIgnoreCase("split_concatenate")) {
			
			// those parameters must be set
			if (data_dir == null)
				Die("Error: data_dir option must be set.");
			if (output_file == null)
				Die("Error: output_file option must be set.");
			
			System.out.println("Parameters");
			System.out.println("----------");
			System.out.println("task:               " + task);
			System.out.println("data_dir:           " + data_dir);
			System.out.println("output_file:        " + output_file);
			
			/* build module network object from several xml files
			* each xml file has only a couple of modules with assigned regulators
			* use specific constructor to reconstruct the module network object
			*/
			ModuleNetwork M = new ModuleNetwork(data_dir,"","");
			
			// Write complete file
			M.writeXML(output_file + ".xml.gz");
		}
		//----------------------------------------------------------------------------
		// fuzzy task: print gene matrix for fuzzy clustering procedure from xml files
		//----------------------------------------------------------------------------
		else if (task.equalsIgnoreCase("fuzzy")) {
			
			// those parameters must be set
			if (data_dir == null)
				Die("Error: data_dir parameter must be set.");
			if (output_file == null)
				Die("Error: output_file option must be set.");
			
			System.out.println("Parameters");
			System.out.println("----------");
			System.out.println("task:               " + task);
			System.out.println("data_dir:           " + data_dir);
			System.out.println("output_file:        " + output_file);
			System.out.println("mir:                " + mir);
			
			if (mir == true) {
				// build module network object from every xml files in a directory, microRNA format
				ModuleNetwork M = new ModuleNetwork(data_dir,"");
				// print out file for tight clustering procedure
				M.printInModules(output_file + ".txt");
			}
			else {
				// build module network object from every xml files in a directory
				ModuleNetwork M = new ModuleNetwork(data_dir,"",false);
				// print out file for tight clustering procedure
				M.printInModules(output_file + ".txt");
			}
		}
		//----------------------------------------------------------------------------
		// print_regulators task: print regulators and their weights
		//----------------------------------------------------------------------------
		else if (task.equalsIgnoreCase("print_regulators")) {
			
			// those parameters must be set
			if (data_dir == null)
				Die("Error: data_dir parameter must be set.");
			if (output_file == null)
				Die("Error: output_file option must be set.");
			
			System.out.println("Parameters");
			System.out.println("----------");
			System.out.println("task:               " + task);
			System.out.println("data_dir:           " + data_dir);
			System.out.println("output_file:        " + output_file);
			System.out.println("mir:                " + mir);
			System.out.println("matlab:             " + matlab);
			System.out.println("all_regulators:     " + all_regulators);
			
			if (mir == true) {
				// build module network object from every xml files in a directory, microRNA format
				ModuleNetwork M = new ModuleNetwork(data_dir,"");
				M.printRegulators(output_file, all_regulators, matlab);
			}
			else {
				// build module network object from every xml files in a directory
				ModuleNetwork M = new ModuleNetwork(data_dir,"",false);
				M.printRegulators(output_file, all_regulators, matlab);
			}
		}
		//----------------------------------------------------------------------------
		// print_random_regulators task: print random regulators and their weights
		//----------------------------------------------------------------------------
		else if (task.equalsIgnoreCase("print_random_regulators")) {
			
			// those parameters must be set
			if (data_dir == null)
				Die("Error: data_dir parameter must be set.");
			if (output_file == null)
				Die("Error: output_file option must be set.");
			
			System.out.println("Parameters");
			System.out.println("----------");
			System.out.println("task:               " + task);
			System.out.println("data_dir:           " + data_dir);
			System.out.println("output_file:        " + output_file);
			System.out.println("mir:                " + mir);
			System.out.println("matlab:             " + matlab);
			
			if (mir == true) {
				// build module network object from every xml files in a directory, microRNA format
				ModuleNetwork M = new ModuleNetwork(data_dir,"");
				M.printRandomRegulators(output_file, matlab);
			}
			else {
				// build module network object from every xml files in a directory
				ModuleNetwork M = new ModuleNetwork(data_dir,"",false);
				M.printRandomRegulators(output_file, matlab);
			}
		}
		//----------------------------------------------------------------------------
		// print_modules: self-explaining, isn't it?
		//----------------------------------------------------------------------------
		else if (task.equalsIgnoreCase("print_modules")) {
			
			// those parameters must be set
			if (data_dir == null)
				Die("Error: data_dir parameter must be set.");
			if (output_file == null)
				Die("Error: output_file option must be set.");
			
			System.out.println("Parameters");
			System.out.println("----------");
			System.out.println("task:               " + task);
			System.out.println("data_dir:           " + data_dir);
			System.out.println("output_file:        " + output_file);
			System.out.println("mir:                " + mir);
			
			if (mir == true) {
				// build module network object from every xml files in a directory, microRNA format
				ModuleNetwork M = new ModuleNetwork(data_dir,"");
				try {
					File output = new File(output_file);
					PrintWriter pw = new PrintWriter(output);
					for (Module mod : M.moduleSet){
						if (!mod.genes.isEmpty())
							for (Gene g : mod.genes){
								String line = g.name + " " + mod.number;
								pw.println(line);
							}
					}
					pw.close();
				} catch (IOException e){
					System.out.println("Error while trying to create the output file");
					System.out.println(e);
					e.printStackTrace();
					System.exit(1);
				}	
			}
			else {
				// build module network object from every xml files in a directory
				ModuleNetwork M = new ModuleNetwork(data_dir,"",false);
				try {
					File output = new File(output_file);
					PrintWriter pw = new PrintWriter(output);
					for (Module mod : M.moduleSet){
						if (!mod.genes.isEmpty())
							for (Gene g : mod.genes){
								String line = g.name + " " + mod.number;
								pw.println(line);
							}
					}
					pw.close();
				} catch (IOException e){
					System.out.println("Error while trying to create the output file");
					System.out.println(e);
					e.printStackTrace();
					System.exit(1);
				}
			}					
		}
		//----------------------------------------------------------------------------
		// go_annotation task: GO annotation of the tight clusters
		//----------------------------------------------------------------------------
		else if (task.equalsIgnoreCase("go_annotation")) {
			
			// those parameters must be set
			if (data_dir == null)
				Die("Error: data_dir parameter must be set.");
			if (output_file == null)
				Die("Error: output_file option must be set.");
			if (go_annot_file == null)
				Die("Error: go_annot_file option must be set.");
			
			System.out.println("Parameters");
			System.out.println("----------");
			System.out.println("task:               " + task);
			System.out.println("data_dir:           " + data_dir);
			System.out.println("output_file:        " + output_file);
			System.out.println("go_annot_file:      " + go_annot_file);
			System.out.println("go_ontology_file:   " + go_ontology_file);
			System.out.println("go_ref_file:        " + go_ref_file);
			System.out.println("go_p_value:         " + go_p_value);
			System.out.println("go_annot_def:       " + go_annot_def);
			System.out.println("go_ontology_def:    " + go_ontology_def);
			System.out.println("map_file:           " + map_file);
			System.out.println("mir:                " + mir);
			
			// load reference list of genes, if any.
			HashSet<String> allNodes = new HashSet<String>();
			if (go_ref_file != null) {
				System.out.println("Loading GO gene reference list..");
				try {
					BufferedReader input = new BufferedReader(new FileReader(new File(go_ref_file)));
					String line;
					while((line = input.readLine()) != null) {
						line.trim();
						allNodes.add(line);
					}
					input.close();
				}
				catch (IOException e) {
					System.out.println("Error while parsing the GO ref file");
					e.printStackTrace();
					System.exit(1);
				}
			}
			
			// eventually load map file for gene names
			HashMap<String, String> map = new HashMap<String, String>();
			if (map_file != null) {
				System.out.println("Loading gene map file..");
				try {
					BufferedReader input = new BufferedReader(new FileReader(new File(map_file)));
					String line;
					while((line = input.readLine()) != null) {
						line.trim();
						String tk[] = line.split("\\t");
						map.put(tk[0], tk[1]);
					}
					input.close();
				}
				catch (FileNotFoundException e) {
					System.out.println("Error: file not found");
					e.printStackTrace();
					System.exit(1);
				}
				catch (IOException e) {
					System.out.println("Error while parsing the map file");
					e.printStackTrace();
					System.exit(1);
				}
			}
			
			if (mir == true) {
				// build module network object from every xml files in a directory, microRNA format
				ModuleNetwork M = new ModuleNetwork(data_dir,"");
				// change gene names if we have a map file
				if (map_file != null)
					M.changeGeneNames(map);
				// create bingo object, annotate modules and write the results to the output_file
				BiNGO go = new BiNGO(M, go_annot_file, go_ontology_file, go_p_value, go_annot_def, go_ontology_def);
				go.GOwrite(output_file, allNodes);
			}
			else {
				// build module network object from every xml files in a directory
				ModuleNetwork M = new ModuleNetwork(data_dir,"",false);
				if (map_file != null)
					M.changeGeneNames(map);
				// create bingo object, annotate modules and write the results to the output_file
				BiNGO go = new BiNGO(M, go_annot_file, go_ontology_file, go_p_value, go_annot_def, go_ontology_def);
				go.GOwrite(output_file, allNodes);
			}
		}
		//----------------------------------------------------------------------------
		// figures task: create eps figures for each module
		//----------------------------------------------------------------------------
		else if (task.equalsIgnoreCase("figures")) {
			
			// those parameters must be set
			if (data_dir == null)
				Die("Error: data_dir parameter must be set.");
			if (top_regulators == null)
				Die("Error: top_regulators option must be set.");
			
			System.out.println("Parameters");
			System.out.println("----------");
			System.out.println("task:               " + task);
			System.out.println("data_dir:           " + data_dir);
			System.out.println("top_regulators:     " + top_regulators);
			System.out.println("use_global_mean:    " + use_global_mean);
			System.out.println("use_regulator_mean: " + use_regulator_mean);
			System.out.println("map_file:           " + map_file);
			System.out.println("cut_level:          " + cut_level);
			System.out.println("mir:                " + mir);
			
			// load map file for gene names
			HashMap<String, String> map = new HashMap<String, String>();
			if (map_file != null) {
				System.out.println("Loading gene map file..");
				try {
					BufferedReader input = new BufferedReader(new FileReader(new File(map_file)));
					String line;
					while((line = input.readLine()) != null) {
						line.trim();
						String tk[] = line.split("\\t");
						map.put(tk[0], tk[1]);
					}
					input.close();
				}
				catch (FileNotFoundException e) {
					System.out.println("Error: file not found");
					e.printStackTrace();
					System.exit(1);
				}
				catch (IOException e) {
					System.out.println("Error while parsing the map file");
					e.printStackTrace();
					System.exit(1);
				}
			}
			
			if (mir == true) {
				// build module network object from every xml files in a directory, microRNA format
				ModuleNetwork M = new ModuleNetwork(data_dir,"");
				// set top regulators from file
				M.setTopRegulators(top_regulators);
				// Calculate mean and sigma for modules and regulators
				M.setModuleMeanSigma();
				// calculate mean and sd for regulators if option is set
				if (use_regulator_mean == true)
					M.setRegulatorMeanSigma();
				// Check if experiments names are defined
				M.checkExperiments();
				// remap gene names eventually
				if (map_file != null)
					M.changeGeneNames(map);
				// cut trees to a certain level
				if (cut_level > 0) {
					for (Module mod : M.moduleSet) {
						for (TreeNode t : mod.hierarchicalTrees) {
							t.testLevel(cut_level);
						}
					}
				}
				// Use global or module mean as the reference for figure color
				M.setGlobalMeanForFigures(use_global_mean);
				// Use reg mean and sd for figures
				M.setRegulatorlMeanForFigures(use_regulator_mean);
				// Now draw an eps figure for each module
				for (Module mod : M.moduleSet) {
					if (mod.genes.size() > 0) {
						if (mod.topRegulators.size() > 0) {
							/*
							 * Some of the hierarchical trees might not be defined, so loop until
							 * we find one that is not null to be able to draw it
							 */
							for (int nb=0;nb < mod.hierarchicalTrees.size(); nb++) {
								TreeNode t = mod.hierarchicalTrees.get(nb);
								mod.hierarchicalTree = t;
								if (mod.hierarchicalTree.nodeStatus.equals("internal")) {
									String file = "module_" + mod.number + "_tree_" + nb;
									System.out.println("Drawing " + file + "...");
									mod.drawEps(file);
									break;
								}
							}
						}
						else {
							//module with no top regulators, so draw it as one leaf
							String file = "module_" + mod.number + "_leaf";
							System.out.println("Drawing " + file + "...");
							mod.drawEps(file);
						}
					}
				}
			}
			else {
				// build module network object from every xml files in a directory
				ModuleNetwork M = new ModuleNetwork(data_dir,"",false);
				M.setTopRegulators(top_regulators);
				M.setModuleMeanSigma();
				if (use_regulator_mean == true)
					M.setRegulatorMeanSigma();
				M.checkExperiments();
				if (map_file != null)
					M.changeGeneNames(map);
				// cut trees to a certain level
				if (cut_level > 0) {
					for (Module mod : M.moduleSet) {
						for (TreeNode t : mod.hierarchicalTrees) {
							t.testLevel(cut_level);
						}
					}
				}
				M.setGlobalMeanForFigures(use_global_mean);
				M.setRegulatorlMeanForFigures(use_regulator_mean);
				
				for (Module mod : M.moduleSet) {
					if (mod.genes.size() > 0) {
						if (mod.topRegulators.size() > 0) {
							/*
							 * Some of the hierarchical trees might not be defined, so loop until
							 * we find one that is not null to be able to draw it
							 */
							for (int nb=0;nb < mod.hierarchicalTrees.size(); nb++) {
								TreeNode t = mod.hierarchicalTrees.get(nb);
								mod.hierarchicalTree = t;
								if (mod.hierarchicalTree.nodeStatus.equals("internal")) {
									String file = "module_" + mod.number + "_tree_" + nb;
									System.out.println("Drawing " + file + " ...");
									mod.drawEps(file);
									break;
								}
							}
						}
						else {
							//module with no top regulators, so draw it as one leaf
							String file = "module_" + mod.number + "_leaf";
							System.out.println("Drawing " + file + " ...");
							mod.drawEps(file);
						}
					}
				}
			}
		}

	
	}
	
	public static void Die (String msg) {
		System.out.println(msg);
		System.exit(1);
	}
	
	public static void printBanner () {
		System.out.println("");
		System.out.println("LeMoNe - Learning Module Networks");
		System.out.println("---------------------------------");
		System.out.println("Version 2.5");
		System.out.println("Copyright (c) 2005-2010 VIB/Ugent");
		System.out.println("Coding crew: Tom Michoel, Anagha Joshi, Eric Bonnet, Steven Maere");
		System.out.println("");
	}
}
