/* LeMoNe - Learning Module Network
 * 
 * Copyright (C) 2005-2008 VIB 
 * 
 * Written by Tom Michoel, Eric Bonnet, Anagha Joshi, Steven Maere
 *  
*/

package be.ugent.psb.ModuleNetwork;

import java.io.*;
//import java.io.FileReader;
import java.text.*;
import java.util.*;
import java.awt.Color;

import org.xml.sax.SAXParseException;

import be.ugent.psb.NetworkTools.*;
import cern.jet.stat.Descriptive;
import cern.colt.list.*;
import cytoscape.data.annotation.OntologyTerm;

public class RunMicro {


	/**
	 * ad-hoc main class to build and analyze network using microRNA + mRNA datasets
	 * 
	 * @author erbon
	 */
	public static void main(String[] args) {
		prost(args[0]);
	}

	public static void analyze_res(String dirname) {

		ModuleNetwork m = new ModuleNetwork(dirname, "");
		//System.out.println("Modulesets: " + m.moduleSets.size());
		
		//to_hugo(m);

		// list random reg scores
//		for (Module mod : m.moduleSet) {
//		    if (!mod.genes.isEmpty())
//		    	for (Gene reg : mod.regulatorWeightsRandom.keySet()){
//                	System.out.println(mod.number + "\t" + reg.name + "\t" + mod.regulatorWeightsRandom.get(reg));
//                }
//		}
	
		// list reg scores
		for (Module mod : m.moduleSet) {
		    if (!mod.genes.isEmpty())
		    	for (Gene reg : mod.regulatorWeights.keySet()){
                	System.out.println(mod.number + "\t" + reg.name + "\t" + mod.regulatorWeights.get(reg));
                }
		}

		
		// old stuff
//		for (Module mod : m.moduleSet){
//			Network net = new Network();
//			for (Gene reg : mod.regulatorWeightsSum.keySet()){
//				Edge ed = new Edge(reg.name, String.format("module_%03d", mod.number), mod.regulatorWeightsSum.get(reg));
//				net.edgeList.add(ed);
//			}
//			Collections.sort(net.edgeList);
//			int c = 0;
//			for (Edge e : net.edgeList){
//				if (c<10)
//				System.out.println(mod.number + "\treg\t" + e.toString());
//				c++;
//			}
//			for (Gene g : mod.genes) {
//				System.out.println(mod.number + "\tgene\t" + g.name);
//			}
//		}
	}
	
	
	public static void ath_go_graphs (String dir) {
		
		ModuleNetwork m = new ModuleNetwork(dir, "",false);
		
		//BiNGO b = new BiNGO(m,"C_elegans_default", "GO_Biological_Process", "0.05", true, true);
		BiNGO b = new BiNGO(m,"A_thaliana_default", "GO_Biological_Process", "0.05", true, true);
		HashSet<String> allNodes = new HashSet<String>();
		b.GOmodules(allNodes);
		//System.exit(1);
		
		// load experiments names
		try {
			BufferedReader input = new BufferedReader(new FileReader(new File("/group/sysbiol/ath_cc/run6/exp/experiments_names")));
			String line;
			int c=0;
			m.conditionSet = new ArrayList<Experiment>();
			while((line = input.readLine()) != null) {
				line.trim();
				Experiment e = new Experiment(line,c);
				m.conditionSet.add(e);
			}
			// build conditionMap
			m.conditionMap = new HashMap<String, Experiment>();
			for (Experiment e : m.conditionSet) {
				m.conditionMap.put(e.name, e);
			}
		}
		catch (IOException e) {
			e.printStackTrace();
			System.exit(1);
		}
		
	
		// compute mean and sigma for each module
		int nbval=0;
		for (Module mod : m.moduleSet) {
			for (Gene g : mod.genes) {
				for (int j=0;j<m.data[g.number].length;j++) {
					if (!Double.isNaN(m.data[g.number][j])) {
						mod.mean += m.data[g.number][j];
						nbval++;
					}
				}
			}
			mod.mean = mod.mean / nbval;
			nbval=0;
		}
		nbval=0;
		for (Module mod : m.moduleSet) {
			for (Gene g : mod.genes) {
				for (int j=0;j<m.data[g.number].length;j++) {
					if (!Double.isNaN(m.data[g.number][j])) {
						mod.sigma += Math.pow(mod.mean - m.data[g.number][j],2);
						nbval++;
					}
				}
			}
			mod.sigma = Math.sqrt(mod.sigma / nbval);
			nbval=0;
			//System.out.println(mod.number+"\t"+mod.mean+"\t"+mod.sigma);
		}
		//compute mean and sigma for each regulators
		for (Gene g : m.regulatorSet) {
			int ct=0;
			for (int c=0;c<m.data[g.number].length;c++) {
				if (!Double.isNaN(m.data[g.number][c])) {
					g.mean += m.data[g.number][c];
					ct++;
				}
			}
			g.mean = g.mean / ct;
		}
		for (Gene g : m.regulatorSet) {
			int ct=0;
			for (int c=0;c<m.data[g.number].length;c++) {
				if (!Double.isNaN(m.data[g.number][c])) {
					g.sigma += Math.pow(g.mean - m.data[g.number][c],2);
					ct++;
				}
			}
			g.sigma = Math.sqrt(g.sigma / ct);
		}
		
		// load top regulators file
		String mapfile = "/group/sysbiol/ath_cc/run6/exp/reg/ath_reg_sc10.txt";
		try {
			BufferedReader input = new BufferedReader(new FileReader(new File(mapfile)));
			String line;
			while((line = input.readLine()) != null) {
				line.trim();
				String[] tk = line.split("\\t");
				int module_number = Integer.parseInt(tk[1]);
				String gene_name = tk[0];

				//System.out.println(m.moduleSet.get(module_number).topRegulators.size());
				m.moduleSet.get(module_number).topRegulators.add(m.geneMap.get(gene_name));
			}
			input.close();
		}
		catch (IOException e) {
			System.out.println("Error while loading top regulators file.");
			e.printStackTrace();
			System.exit(1);
		}
		
//		for (Module mod : m.moduleSet) {
//			if (mod.topRegulators.size()>0) {
//				for (Gene g : mod.topRegulators)
//					System.out.println(mod.number + "\t" + g.name);
//			}
//		}
		
		// load map AT to common names  
		HashMap<String,String> ref = new HashMap<String, String>();
		try {
			BufferedReader input = new BufferedReader(new FileReader(new File("/group/sysbiol/ath_cc/run6/exp/ath_common_names")));
			String line;
			while((line = input.readLine()) != null) {
				line.trim();
				String[] tk = line.split("\\t");
				ref.put(tk[0], tk[1]);
			}
			input.close();
		}
		catch (IOException e) {
			e.printStackTrace();
			System.exit(1);
		}

		// replace AT names with common names
		for (Gene g : m.geneSet) {
			String key = g.name;
			g.name = ref.get(key);
		}
		
		//for (Gene g : m.geneSet)
		//	System.out.println(g.name);
		
			
		for (Module mod : m.moduleSet) {
			int c=0;
			//for (TreeNode n : mod.hierarchicalTrees) {
			TreeNode n = mod.hierarchicalTrees.get(0);
				mod.hierarchicalTree = n;
				for (TreeNode node : mod.hierarchicalTree.getInternalNodes()){
					Collections.sort(node.testSplits);
					node.regulationSplit = node.testSplits.get(0);
				}
				if (mod.hierarchicalTree.nodeStatus.equals("internal")) {
					String file = String.format("module_%03d-%d", mod.number, c);
					System.out.println(file);
					//mod.draw(file);
					mod.drawGOEps(file);
				}
				c++;
			//}	
		}
	} // end ath_go_graphs	
	
	public static void vamei_data (String dir) {
		
		ModuleNetwork m = new ModuleNetwork(dir, "",false);
		
		//BiNGO b = new BiNGO(m,"C_elegans_default", "GO_Biological_Process", "0.05", true, true);
		BiNGO b = new BiNGO(m,"annot", "GO_Biological_Process", "0.05", false, true);
		HashSet<String> allNodes = new HashSet<String>();
		b.GOmodules(allNodes);
		//System.exit(1);
		
		// load experiments names
		try {
			BufferedReader input = new BufferedReader(new FileReader(new File("/group/sysbiol/erbon/vamei/exp")));
			String line;
			int c=0;
			m.conditionSet = new ArrayList<Experiment>();
			while((line = input.readLine()) != null) {
				line.trim();
				Experiment e = new Experiment(line,c);
				m.conditionSet.add(e);
			}
			// build conditionMap
			m.conditionMap = new HashMap<String, Experiment>();
			for (Experiment e : m.conditionSet) {
				m.conditionMap.put(e.name, e);
			}
		}
		catch (IOException e) {
			e.printStackTrace();
			System.exit(1);
		}
		
		
//		for (Experiment e : m.conditionSet) {
//			System.out.println(e.name);
//		}
		
		// compute mean and sigma for each module
		int nbval=0;
		for (Module mod : m.moduleSet) {
			for (Gene g : mod.genes) {
				for (int j=0;j<m.data[g.number].length;j++) {
					if (!Double.isNaN(m.data[g.number][j])) {
						mod.mean += m.data[g.number][j];
						nbval++;
					}
				}
			}
			mod.mean = mod.mean / nbval;
			nbval=0;
		}
		nbval=0;
		for (Module mod : m.moduleSet) {
			for (Gene g : mod.genes) {
				for (int j=0;j<m.data[g.number].length;j++) {
					if (!Double.isNaN(m.data[g.number][j])) {
						mod.sigma += Math.pow(mod.mean - m.data[g.number][j],2);
						nbval++;
					}
				}
			}
			mod.sigma = Math.sqrt(mod.sigma / nbval);
			nbval=0;
			//System.out.println(mod.number+"\t"+mod.mean+"\t"+mod.sigma);
		}
		//compute mean and sigma for each regulators
		for (Gene g : m.regulatorSet) {
			int ct=0;
			for (int c=0;c<m.data[g.number].length;c++) {
				if (!Double.isNaN(m.data[g.number][c])) {
					g.mean += m.data[g.number][c];
					ct++;
				}
			}
			g.mean = g.mean / ct;
		}
		for (Gene g : m.regulatorSet) {
			int ct=0;
			for (int c=0;c<m.data[g.number].length;c++) {
				if (!Double.isNaN(m.data[g.number][c])) {
					g.sigma += Math.pow(g.mean - m.data[g.number][c],2);
					ct++;
				}
			}
			g.sigma = Math.sqrt(g.sigma / ct);
		}
		
		// load top regulators file
		String mapfile = "/group/sysbiol/vamei/workspace/lemone/data/lemone_nov08/best5reg_matrixclustWS195";
		try {
			BufferedReader input = new BufferedReader(new FileReader(new File(mapfile)));
			String line;
			while((line = input.readLine()) != null) {
				line.trim();
				String[] tk = line.split("\\t");
				int module_number = Integer.parseInt(tk[0]);
				String gene_name = tk[1];

				//System.out.println(m.moduleSet.get(module_number).topRegulators.size());
				m.moduleSet.get(module_number).topRegulators.add(m.geneMap.get(gene_name));
			}
			input.close();
		}
		catch (IOException e) {
			System.out.println("Error while loading top regulators file.");
			e.printStackTrace();
			System.exit(1);
		}
		
//		for (Module mod : m.moduleSet) {
//			if (mod.topRegulators.size()>0) {
//				for (Gene g : mod.topRegulators)
//					System.out.println(mod.number + "\t" + g.name);
//			}
//		}
		
	
			
		for (Module mod : m.moduleSet) {
			int c=0;
			//for (TreeNode n : mod.hierarchicalTrees) {
			TreeNode n = mod.hierarchicalTrees.get(0);
				mod.hierarchicalTree = n;
				for (TreeNode node : mod.hierarchicalTree.getInternalNodes()){
					Collections.sort(node.testSplits);
					node.regulationSplit = node.testSplits.get(0);
				}
				if (mod.hierarchicalTree.nodeStatus.equals("internal")) {
					String file = String.format("module_%03d-%d", mod.number, c);
					System.out.println(file);
					//mod.draw(file);
					mod.drawGOEps(file);
				}
				c++;
			//}	
		}
	}

	public static void mir_learn_reg_for_tight_clusters (String filename) {

		filename = filename + ".xml.gz";

		// Set data directory
		String dataDir = "/group/sysbiol/java/test_data";

		// complete dataset
		String geneFile =      "lu_2005_mRNA.txt";
		//String clusterFile =   "lu_2005_tight_cluster_043_tight_exp_clusters.txt";
		String clusterFile =   "lu_tight_cluster_025.txt";
		String mirFile =       "lu_2005_miRNA.txt";
		String regFile =       "lu_2005_regulators.txt";

		// Set initial number of clusters
		int numclust = 99;

		// Set normal-gamma prior paramaters
		double lambda0 = 0.1;
		double mu0 =     0.0;
		double alpha0 =  0.1;
		double beta0 =   0.1;

		double scoregain = 0.005;
		boolean useBHCscore = true;

		int numRunsExpts = 1;
		int numStepsExpts = 1100;
		int burnInExpts = 100;
		int sampleStepExpts = 10;

		//int numStepsExpts = 2;
		//int burnInExpts = 1;
		//int sampleStepExpts = 2;

		//int cutLevel = 2;
		//int minSizeExpt = 10;
		//int minSizeExpt = 16;
		//double signiFact = 1;

		double betaReg = 75;
		int numReg = 100;
		//int numReg = 1;

		HashMap<String, String> fake = new HashMap<String, String>();

		// print parameters in the log file
		System.out.println("data file: " + geneFile);
		System.out.println("cluster file: " + clusterFile);
		System.out.println("mir file: " + mirFile);
		System.out.println("regulators: " + regFile);
		System.out.println("Nb clusters: " + numclust);
		System.out.println("scoregain: " + scoregain);
		System.out.println("useBHCscore: " + useBHCscore);
		System.out.println("numRunsExpts: " + numRunsExpts);
		System.out.println("numStepsExpts: " + numStepsExpts);
		System.out.println("burnInExpts: " + burnInExpts);
		System.out.println("sampleStepExpts: " + sampleStepExpts);
		System.out.println("betaReg: " + betaReg);
		System.out.println("numReg: " + numReg);

		// Construct module network
		ModuleNetwork M = new ModuleNetwork(dataDir, geneFile, mirFile, numclust, clusterFile, regFile, lambda0, mu0, alpha0, beta0, fake);

		M.dataType = M.dataType.AFFY;

		// rescale scoregain
		scoregain = scoregain * M.numGenes;

		M.gibbsSamplerExpts(numRunsExpts, numStepsExpts, burnInExpts, sampleStepExpts, scoregain, useBHCscore);

		//M.makeConsensusTrees(cutLevel, minSizeExpt, signiFact);

		M.assignRegulatorsNoAcyclStoch(betaReg, numReg);

		M.writeMicroXML(filename);
	}

	// just print the list of genes for all modules and all module sets
	public static void ath_print_modules(String dirname) {
		System.out.println(dirname);
		ModuleNetwork m = new ModuleNetwork(dirname,"",false);
		for (int k=0; k<m.moduleSets.size(); k++){
			for (Module mod : m.moduleSets.get(k)){
				for (Gene g : mod.genes) {
					System.out.println(k + "\t" + mod.number + "\t" + g.name);
				}
			}
		}
	}
	/**
	 * change gene names from probeset to HUGO names
	 * 
	 * @param m
	 */
	public static void to_hugo (ModuleNetwork m) {

		//String mapfile = "/Users/eric/work/probe2hugo.mir.map"; 
		String mapfile = "/group/sysbiol/mirna/runs/stock/run_eric_11-2007/probe2hugo.map.complete";

		HashMap<String,String> ref = new HashMap<String, String>();

		// load cluster list 
		try {
			BufferedReader input = new BufferedReader(new FileReader(new File(mapfile)));
			String line;
			while((line = input.readLine()) != null) {
				line.trim();
				String[] tk = line.split("\\t");
				ref.put(tk[0], tk[1]);
			}
			input.close();
		}
		catch (IOException e) {
			e.printStackTrace();
			System.exit(1);
		}

		// replace probe names with HUGO codes
		for (Gene g : m.geneSet) {
			String key = g.name;
			g.name = ref.get(key);
		}
	}


	public static void hugo_graphs(String dirname) {

		ModuleNetwork m = new ModuleNetwork(dirname, "");

		for (Module mod : m.moduleSet)
			mod.setRegConditions();

		//Iterator<Gene> it =  m.moduleSet.get(29).regConditions.keySet().iterator();
		//ArrayList tmp = m.moduleSet.get(29).regConditions.get(m.geneMap.get("Hu35KsubA/RC_AA609113_at"));

		//for (int i=0;i<tmp.size();i++)
		//	System.out.println(tmp.get(i));

		// load top regulator list and add them to the modules 
		//String mapfile = "/group/sysbiol/mirna/runs/stock/run_eric_11-2007/run_tight_clusters_01-2008/run2_top10_reg.mod_number";
		//String mapfile = "/group/sysbiol/mirna/runs/stock/run_eric_11-2007/run_tight_clusters_01-2008/tight_015/top_reg_10";
		//String mapfile = "/group/sysbiol/mirna/runs/stock/run_eric_11-2007/run_tight_clusters_01-2008/tight_020/top_reg_10";
		//String mapfile = "/group/sysbiol/mirna/runs/stock/run_eric_11-2007/run_tight_clusters_01-2008/tight_025/top_reg_10";

		//String mapfile = "/group/sysbiol/mirna/runs/stock/run_eric_11-2007/run_tight_clusters_01-2008/run2_reg_298";
		String mapfile = "/group/sysbiol/mirna/runs/stock/run_eric_11-2007/run_tight_clusters_01-2008/run2_regw_probes_sort_209";

		try {
			BufferedReader input = new BufferedReader(new FileReader(new File(mapfile)));
			String line;
			while((line = input.readLine()) != null) {
				line.trim();
				String[] tk = line.split("\\t");
				// old file format
				//int module_number = Integer.parseInt(tk[4]);
				//String gene_name = tk[0];
				int module_number = Integer.parseInt(tk[0]);
				String gene_name = tk[1];

				//System.out.println(m.moduleSet.get(module_number).topRegulators.size());
				m.moduleSet.get(module_number).topRegulators.add(m.geneMap.get(gene_name));
			}
			input.close();
		}
		catch (IOException e) {
			System.out.println("Error while loading top regulators file.");
			e.printStackTrace();
			System.exit(1);
		}

		// load microRNA seed targets
		String seed_file = "/group/sysbiol/mirna/runs/stock/run_eric_11-2007/seed_data";
		try {
			BufferedReader input = new BufferedReader(new FileReader(new File(seed_file)));
			String line;
			while((line = input.readLine()) != null) {
				line.trim();
				String[] tk = line.split("\\t");
				String mir = tk[1];
				String probe = tk[2];
				Gene my_probe = m.geneMap.get(probe);
				Gene my_mir = m.geneMap.get(mir);
				// be careful, not all mir and probeset are defined in the dataset
				if (my_mir != null && my_probe != null)
					my_mir.seedTargets.add(my_probe);
			}
			input.close();
		}
		catch (IOException e) {
			System.out.println("Error while loading seed data file.");
			e.printStackTrace();
			System.exit(1);
		}

		// load microRNA predicted targets
		String pred_file = "/group/sysbiol/mirna/runs/stock/run_eric_11-2007/pred_data";
		try {
			BufferedReader input = new BufferedReader(new FileReader(new File(pred_file)));
			String line;
			while((line = input.readLine()) != null) {
				line.trim();
				String[] tk = line.split("\\t");
				String mir = tk[0];
				String probe = tk[1];
				Gene my_probe = m.geneMap.get(probe);
				Gene my_mir = m.geneMap.get(mir);
				//System.out.println(mir + "\t" + my_mir.name + "\t" + probe + "\t" + my_probe.name);
				// be careful, not all mir and probeset are defined in the dataset
				if (my_mir != null && my_probe != null)
					my_mir.predTargets.add(my_probe);
			}
			input.close();
		}
		catch (IOException e) {
			System.out.println("Error while loading predicted targets data file.");
			e.printStackTrace();
			System.exit(1);
		}

		// replace probesets with hugo names
		to_hugo(m);

		// do GO annotation for each module
		BiNGO b = new BiNGO(m,"H_sapiens_default", "GO_Biological_Process", "0.05", true, true);
		HashSet<String> allNodes = new HashSet<String>();
		// load reference list of genes
		try {
			BufferedReader input = new BufferedReader(new FileReader(new File("/group/sysbiol/mirna/runs/stock/run_eric_11-2007/all_gene_hugo_list")));
			String line;
			while((line = input.readLine()) != null) {
				line.trim();
				allNodes.add(line);
			}
			input.close();
		}
		catch (IOException e) {
			e.printStackTrace();
			System.exit(1);
		}

		b.GOmodules(allNodes);
		//b.GOwrite("mygo.txt", allNodes);
		//System.exit(1);

		// annotate experiments
		try {
			BufferedReader input = new BufferedReader(new FileReader(new File("/group/sysbiol/mirna/lu_et_al_nature_2005/conditions.txt")));
			String line;
			while((line = input.readLine()) != null) {
				line.trim();
				String[] tk = line.split("\\t");
				String exp_name = tk[0];
				String state = tk[1];
				String tissue = tk[2];
				int rank = Integer.parseInt(tk[3]);
				m.conditionMap.get(exp_name).state = state;
				m.conditionMap.get(exp_name).tissue = tissue;
				m.conditionMap.get(exp_name).rank = rank;
				if (tissue.equals("bladder"))
					m.conditionMap.get(exp_name).col = new Color(0,84,166); // blue
				if (tissue.equals("colon"))
					m.conditionMap.get(exp_name).col = new Color(184,219,124); // green
				if (tissue.equals("prostate"))
					m.conditionMap.get(exp_name).col = new Color(247,163,10); // orange
				if (tissue.equals("uterus"))
					m.conditionMap.get(exp_name).col = Color.MAGENTA;
				if (tissue.equals("breast"))
					m.conditionMap.get(exp_name).col = Color.BLACK;
				if (tissue.equals("kidney"))
					m.conditionMap.get(exp_name).col = new Color(255,245,0); // yellow
				if (tissue.equals("lung"))
					m.conditionMap.get(exp_name).col = Color.PINK;
				if (tissue.equals("melanoma"))
					m.conditionMap.get(exp_name).col = Color.CYAN;
				if (tissue.equals("mesothelioma"))
					m.conditionMap.get(exp_name).col = new Color(160,206,241); // pastel blue
				if (tissue.equals("ovary"))
					m.conditionMap.get(exp_name).col = new Color(158,107,38); // brown
				if (tissue.equals("pancreas"))
					m.conditionMap.get(exp_name).col = new Color(210,9,211); // purple
			}
			input.close();
		}
		catch (IOException e) {
			e.printStackTrace();
			System.exit(1);
		}

		// compute mean and sigma for each module
		int nbval=0;
		for (Module mod : m.moduleSet) {
			for (Gene g : mod.genes) {
				for (int j=0;j<m.data[g.number].length;j++) {
					if (!Double.isNaN(m.data[g.number][j])) {
						mod.mean += m.data[g.number][j];
						nbval++;
					}
				}
			}
			mod.mean = mod.mean / nbval;
			nbval=0;
		}
		nbval=0;
		for (Module mod : m.moduleSet) {
			for (Gene g : mod.genes) {
				for (int j=0;j<m.data[g.number].length;j++) {
					if (!Double.isNaN(m.data[g.number][j])) {
						mod.sigma += Math.pow(mod.mean - m.data[g.number][j],2);
						nbval++;
					}
				}
			}
			mod.sigma = Math.sqrt(mod.sigma / nbval);
			nbval=0;
			//System.out.println(mod.number+"\t"+mod.mean+"\t"+mod.sigma);
		}

		//compute mean and sigma for each regulators
		for (Gene g : m.regulatorSet) {
			int ct=0;
			for (int c=0;c<m.data[g.number].length;c++) {
				if (!Double.isNaN(m.data[g.number][c])) {
					g.mean += m.data[g.number][c];
					ct++;
				}
			}
			g.mean = g.mean / ct;
		}
		for (Gene g : m.regulatorSet) {
			int ct=0;
			for (int c=0;c<m.data[g.number].length;c++) {
				if (!Double.isNaN(m.data[g.number][c])) {
					g.sigma += Math.pow(g.mean - m.data[g.number][c],2);
					ct++;
				}
			}
			g.sigma = Math.sqrt(g.sigma / ct);
		}

//		m.dataType = m.dataType.AFFY;

//		for (Module mod : m.moduleSet) {
//		if (mod.topRegulators.size()>0) {
//		for (Gene reg : mod.topRegulators) {
//		System.out.println(mod.number + "\t" + reg.name);
//		}
//		}
//		}

//		for (Gene g : m.mirSet) {
//		for (Gene t : g.seedTargets) {
//		System.out.println(g.name + "\t" + t.name);
//		}
//		}
//		System.exit(1);		

		// draw figures for all module sets, taking the best regulator for each tree node;
//		for (int k=0; k<m.moduleSets.size(); k++){
//		System.out.println(m.moduleSets.get(k).size());
//		String dir = String.format("figs/moduleSet_%02d/", k);
//		File figdir = new File(dir);
//		figdir.mkdirs();
//		for (Module mod : m.moduleSets.get(k)){
//		System.out.println(mod.number + "\t" + mod.hierarchicalTrees.size());
//		int c = 1;
//		// for each regulation tree
//		for (TreeNode n : mod.hierarchicalTrees) {
//		mod.hierarchicalTree = n;
//		for (TreeNode node : mod.hierarchicalTree.getInternalNodes()){
//		Collections.sort(node.testSplits);
//		node.regulationSplit = node.testSplits.get(0);
//		}
//		if (mod.hierarchicalTree.nodeStatus.equals("internal")) {
//		String file = String.format(dir + "module_%03d-%d.png", mod.number, c);
//		System.out.println(file);
//		mod.drawConditions(file);
//		}
//		c++;
//		}
//		}
//		}


		/* ----- draw figures for main moduleset and all reg tree ----- */

		for (Module mod : m.moduleSet){
			//String dir = String.format("figures_module_%02d/", mod.number);
			//File d = new File(dir);
			//d.mkdirs();
			int c = 1;
			TreeNode n = mod.hierarchicalTrees.get(0);
			//for (TreeNode n : mod.hierarchicalTrees) {
				mod.hierarchicalTree = n;
//				for (TreeNode node : mod.hierarchicalTree.getInternalNodes()){
//					Collections.sort(node.testSplits);
//					node.regulationSplit = node.testSplits.get(0);
//				}
				if (mod.hierarchicalTree.nodeStatus.equals("internal")) {
					//String file = String.format(dir + "module_%02d-%d", mod.number, c);
					String file = String.format("module_%02d-%d", mod.number, c);
					System.out.println(file);
					mod.drawMirEps(file);
				}
				c++;
			//}
		}


		/* --------   draw modules based on a list -------- */

//		ArrayList<Integer> modlist  = new ArrayList<Integer>();
//
//		//modlist.add(15);
//		//modlist.add(17);
////		modlist.add(18);
////		modlist.add(25);
////		modlist.add(29);
//		//modlist.add(30);
//		//modlist.add(31);
//		//modlist.add(33);
//		modlist.add(35);
//		//modlist.add(79);
//		modlist.add(94);
//
//		// draw figures for main moduleset and all reg trees
//		for (int i : modlist) {
//			Module mod = m.moduleSet.get(i);
//			int c = 1;
//			String dir = String.format("figures_module_%02d/", mod.number);
//			File d = new File(dir);
//			d.mkdirs();
//			for (TreeNode n : mod.hierarchicalTrees) {
//				// just plot the first tree
//				//ArrayList<TreeNode> tmp = new ArrayList<TreeNode>(); 
//				//tmp.add(mod.hierarchicalTrees.get(0));
//				//for (TreeNode n : tmp){
//				mod.hierarchicalTree = n;
//				//for (TreeNode node : mod.hierarchicalTree.getInternalNodes()){
//				//	Collections.sort(node.testSplits);
//				//	node.regulationSplit = node.testSplits.get(0);
//				//}
//				if (mod.hierarchicalTree.nodeStatus.equals("internal")) {
//					//String file = String.format("module_%02d", mod.number);
//					String file = String.format(dir + "module_%02d-%d", mod.number, c);
//					System.out.println(file);
//					mod.drawMirEps(file);
//				}
//				c++;
//			}
//		}
	}

	public static void graphs (String filename) {

		ModuleNetwork M = new ModuleNetwork(filename, "");
		M.dataType = M.dataType.AFFY;

		// draw figures for all module sets, taking the best regulator for each tree node;
		// a new directory is created for each module set
		for (int k=0; k<M.moduleSets.size(); k++){
			System.out.println(M.moduleSets.get(k).size());
			String dir = String.format("figs/moduleSet_%02d/", k);
			File figdir = new File(dir);
			figdir.mkdirs();
			for (Module mod : M.moduleSets.get(k)){
				System.out.println(mod.number + "\t" + mod.hierarchicalTrees.size());
				int c = 1;
				// for each regulation tree
				for (TreeNode n : mod.hierarchicalTrees) {
					mod.hierarchicalTree = n;
					for (TreeNode node : mod.hierarchicalTree.getInternalNodes()){
						Collections.sort(node.testSplits);
						node.regulationSplit = node.testSplits.get(0);
					}
					if (mod.hierarchicalTree.nodeStatus.equals("internal")) {
						String file = String.format(dir + "module_%03d-%d.png", mod.number, c);
						System.out.println(file);
						mod.drawConditions(file);
					}
					c++;
				}
			}
		}
	}

	public static void ath_graphs (String dirname) {
		ModuleNetwork m = new ModuleNetwork(dirname);
		for (Module mod : m.moduleSet) {
			System.out.println(mod.number + " " + mod.hierarchicalTrees.size());
		}
	}

	public static void modules (String filename) {

		ModuleNetwork m = new ModuleNetwork(filename);

		int cond = m.data[0].length;

		for (int k=0;k<m.moduleSets.size();k++) {
			for (Module mod : m.moduleSets.get(k)) {
				//System.out.println(mod.number + "\t" + mod.genes.size());

				int max = mod.genes.size();

				ArrayList<Double> res = new ArrayList<Double>(1);

				for (int i=0;i<max;i++) {
					for (int j=0;j<i;j++) {
						double[] d1 = new double[cond];
						double[] d2 = new double[cond];

						for (int g=0;g<cond;g++) {
							d1[g] = m.data[mod.genes.get(i).number][g];
							d2[g] = m.data[mod.genes.get(j).number][g];
						}

						DoubleArrayList tmp1 = new DoubleArrayList(d1);
						DoubleArrayList tmp2 = new DoubleArrayList(d2);

						double sum1 = Descriptive.sum(tmp1);
						double sum2 = Descriptive.sum(tmp2);

						double sq1 = Descriptive.sumOfSquares(tmp1);
						double sq2 = Descriptive.sumOfSquares(tmp2);

						double var1 = Descriptive.variance(d1.length, sum1, sq1);
						double var2 = Descriptive.variance(d2.length, sum2, sq2);

						double sd1 = Descriptive.standardDeviation(var1);
						double sd2 = Descriptive.standardDeviation(var2);

						double cor = Descriptive.correlation(tmp1, sd1, tmp2, sd2);

						res.add(Math.abs(cor));
					}
				}

				DoubleArrayList dbl = new DoubleArrayList();

				for (int z=0;z<res.size();z++) {
//					if (mod.number == 38) {
//	System.out.println(res.get(z));
//					}
					if (Double.isNaN(res.get(z)) == false)
						dbl.add(res.get(z));
				}

				double mean = Descriptive.mean(dbl);
				double sd = Descriptive.standardDeviation(Descriptive.variance(dbl.size(),Descriptive.sum(dbl),Descriptive.sumOfSquares(dbl)));
				if (Double.isNaN(mean) == false)
					System.out.println(mod.number + "\t" + mod.genes.size() + "\t" + mean + "\t" + sd);
			}
		}

	}

	public static void test_clustering (String[] args) {

		String filename = args[0] + args[1] + ".xml.gz";
		int num = Integer.parseInt(args[1]);

		System.out.println(num);

		String dataDir = "/group/biocomp/projects/segal/java/test_data/";
		//String expressiondata = "lu_2005_mRNA.txt";
		String expressiondata = "lu_2005_mRNA_sd75.txt";
		String miRNAexpressiondata = "lu_2005_miRNA.txt";
		String regulatordata = "lu_2005_regulators.txt";

		double lambda0 = 0.1;
		double mu0 = 0.0;
		double alpha0 = 0.1;
		double beta0 = 0.1;

		double scoregain = 0.001;
		int numRunsGenes = 1;
		int numStepsGenes = 100;
		int burnInGenes = 50;
		int sampleStepGenes = 100;

		//test parameters
		//int numStepsGenes = 2;
		//int burnInGenes = 1;
		//int sampleStepGenes = 2;

		int initNumClust = num;

		boolean useBHCscore = true;

		// Create ModuleNetwork object
		ModuleNetwork M = new ModuleNetwork(dataDir, expressiondata, miRNAexpressiondata, regulatordata, lambda0, mu0, alpha0, beta0);

		// rescale scoregain
		scoregain = scoregain * M.numGenes;

		// Gibbs sample different module sets with one tree per module
		M.gibbsSamplerGenes(initNumClust, numRunsGenes, numStepsGenes, burnInGenes, sampleStepGenes, scoregain, useBHCscore);

		System.out.println("Clusters: " + M.moduleSet.size());

		M.writeMicroXML(filename);
	}

	public static void tight (String dir) {
		//ModuleNetwork m = new ModuleNetwork(dir,"",false);
		ModuleNetwork m = new ModuleNetwork(dir,"");
		m.printInModules("tight_clusters.txt");
	}

	public static void stock (String filename) {

		/**
		 * Build a miRNA::mRNA network using stochastic methods
		 */

		filename = filename + ".xml.gz";

		String dataDir = "/group/sysbiol/java/test_data/";
		String genefile = "lu_2005_mRNA.txt";
		String mirfile = "lu_2005_miRNA.txt";
		String regfile = "lu_2005_regulators.txt";

		double lambda0 = 0.1;
		double mu0 = 0.0;
		double alpha0 = 0.1;
		double beta0 = 0.1;

		double scoregain = 0.005;
		//double scoregain = 0.05;
		int numRunsGenes = 1;
		int numStepsGenes = 100;
		int burnInGenes = 50;
		int sampleStepGenes = 100;
		int initNumClust = 6000;
		//int initNumClust = 110;

		// test parameters
		//int numRunsGenes = 1;
		//int numStepsGenes = 2;
		//int burnInGenes = 1;
		//int sampleStepGenes = 2;

		boolean useBHCscore = true;

//		boolean gibbsSampleExpts = true;
//		int numRunsExpts = 1;
//		int numStepsExpts = 1100;
//		int burnInExpts = 100;
//		int sampleStepExpts = 100;

		// test parameters
		//int numRunsExpts = 1;
		//int numStepsExpts = 2;
		//int burnInExpts = 1;
		//int sampleStepExpts = 2;

//		boolean makeConsensusTree = true;
//		int cutLevel = 1;
//		int minSizeExpt = 10;
//		double signiFact = 1;

//		double betaReg = 75;
//		int numReg = 10;


		// print parameters in the log file
		System.out.println("######## Parameters ##########");
		System.out.println("data directory: " + dataDir);
		System.out.println("data file: " + genefile);
		System.out.println("mir file: " + mirfile);
		System.out.println("regulators: " + regfile);
		System.out.println("Initial nb. clusters: " + initNumClust);
		System.out.println("scoregain: " + scoregain);
		System.out.println("useBHCscore: " + useBHCscore);
		System.out.println("##############################");


		// Create ModuleNetwork object
		ModuleNetwork M = new ModuleNetwork(dataDir, genefile, mirfile, regfile, lambda0, mu0, alpha0, beta0);

		// rescale scoregain
		scoregain = scoregain * M.numGenes;

		// Gibbs sample different module sets with one tree per module
		M.gibbsSamplerGenes(initNumClust, numRunsGenes, numStepsGenes, burnInGenes, sampleStepGenes, scoregain, useBHCscore);

		// if Gibbs sample parameters for experiments, generate multiple trees
		// for the different module sets
		//M.gibbsSamplerExpts(numRunsExpts, numStepsExpts, burnInExpts, sampleStepExpts, scoregain, useBHCscore);

//		if (makeConsensusTree)
//		M.makeConsensusTrees(cutLevel, minSizeExpt, signiFact);

		//M.assignRegulatorsNoAcyclStoch(betaReg, numReg);

		System.out.println("Final nb clusters: " + M.moduleSet.size());

		M.writeMicroXML(filename);

	}

	public static void bingo (String dir, String refset) {

		ModuleNetwork m = new ModuleNetwork(dir,"");

//		#BiNGO parameters
//		BiNGO=true
//		signif_def=0.05
//		annotation_file_def=A_thaliana_default
//		ontology_file_def=GO_Biological_Process
//		annotation_default=true
//		ontology_default=true

		to_hugo(m);

//		for (Module mod : m.moduleSet) {
//		for (Gene g : mod.genes) {
//		System.out.println(mod.number + "\t" + g.name);
//		}	
//		}
//		for (Gene g : m.geneSet) {
//		System.out.println(g.name);
//		}

		//BiNGO b = new BiNGO(m,"A_thaliana_default", "GO_Biological_Process", "0.05", true, true);
		BiNGO b = new BiNGO(m,"H_sapiens_default", "GO_Biological_Process", "0.05", true, true);

		String filename = "toto";
		HashSet<String> allNodes = new HashSet<String>();

		// load reference list of genes
		try {
			BufferedReader input = new BufferedReader(new FileReader(new File(refset)));
			String line;
			while((line = input.readLine()) != null) {
				line.trim();
				allNodes.add(line);
			}
			input.close();
		}
		catch (IOException e) {
			e.printStackTrace();
			System.exit(1);
		}

		//b.GOwrite(filename, allNodes);
		b.GOmodules(allNodes);

//		for (Gene g : m.geneSet) {
//		for (OntologyTerm o : g.GOcat) {
//		System.out.println(g.name+"\t"+o.getName());
//		}
//		}


		for (Module mod : m.moduleSet) {
			for (OntologyTerm o : mod.GOsuper) {
				System.out.println(mod.number + "\t" + o.getName());
			}
		}
	}

	// compute bingo stats from a [ gene - cluster number ] file
	// file1 = cluster file
	// file2 = full list of genes (reference to compute p-values)
//	public static void bingo_for_tight (String file1, String file2) {

//	ArrayList<ArrayList<String>> data = new ArrayList<ArrayList<String>>();
//	ArrayList<String> f = new ArrayList<String>();
//	HashSet<String> num_modules = new HashSet<String>();
//	ArrayList<String> refset = new ArrayList<String>();

//	// load cluster list 
//	try {
//	BufferedReader input = new BufferedReader(new FileReader(new File(file1)));
//	String line;
//	while((line = input.readLine()) != null) {
//	line.trim();
//	f.add(line);
//	String[] tk = line.split("\\s");
//	num_modules.add(tk[1]);
//	}
//	input.close();
//	}
//	catch (IOException e) {
//	e.printStackTrace();
//	System.exit(1);
//	}

//	// initialize array with the number of clusters
//	for (int i=0;i<num_modules.size();i++) {
//	data.add(new ArrayList<String>());
//	}
//	// now fill the data structure
//	for (int i=0;i<f.size();i++) {
//	String tk[] = f.get(i).split("\\s");
//	int num = Integer.parseInt(tk[1]);
//	// start at 1 in matlab
//	num--;
//	data.get(num).add(tk[0]);
//	}

////	for (int i=0;i<data.size();i++) {
////	for (int j=0;j<data.get(i).size();j++) {
////	System.out.println(i + "\t" + data.get(i).get(j));
////	}
////	}

//	// load reference list of genes
//	try {
//	BufferedReader input = new BufferedReader(new FileReader(new File(file2)));
//	String line;
//	while((line = input.readLine()) != null) {
//	line.trim();
//	refset.add(line);
//	}
//	input.close();
//	}
//	catch (IOException e) {
//	e.printStackTrace();
//	System.exit(1);
//	}

//	//BiNGO b = new BiNGO(data, refset, "notused", file1, "A_thaliana_default", "GO_Biological_Process", "0.05", true, true);
//	BiNGO b = new BiNGO(data, refset, "notused", file1, "H_sapiens_default", "GO_Biological_Process", "0.05", true, true);

//	// print out selected GO categories for each module
////	for (Integer num : b.GOmod.keySet()) {
////	for (int i=0;i<b.GOmod.get(num).size();i++) {
////	System.out.println(num + "\t" + b.GOmod.get(num).get(i).getName());
////	}
////	}
//	}

	public static void get_score(String filename) {
		ModuleNetwork m = new ModuleNetwork();
		m.parameters = new HashMap<String,String>();

		double s=0.0;

		try {
			m.fromMicroXML(filename);
		}
		catch (IOException e) {
			e.printStackTrace();
		}
		catch (SAXParseException e1) {
			e1.printStackTrace();
		}
		for (Module mod : m.moduleSets.get(0)) {
			s = s + mod.hierarchicalTree.bayesianScore();
		}
		System.out.println(s);
		System.out.println(s / m.numGenes);
	}

	public static void analyze(String dir){

		System.out.println("Start.");

		System.out.println("Loading data...");
		System.out.println(dir);

		// build network using microRNA data constructor
		ModuleNetwork m = new ModuleNetwork(dir,"notused");

		System.out.println("Modulesets: " + m.moduleSets.size());

		/*
		for (ArrayList<Module>  modset : m.moduleSets) {
			for (Module mo : modset)
				System.out.println(mo.number + "\t" + mo.hierarchicalTrees.size());
		}
		 */

		System.out.println("Building analyzer...");
		ModuleNetworkAnalyzer a = new ModuleNetworkAnalyzer(m,true);

		System.out.println(a.geneNetworkMax.numNodes);

		System.out.println("Sum edges :" + a.geneNetworkSum.numEdges);
		System.out.println("Sum nodes :" + a.geneNetworkSum.numNodes);

		System.out.println("Max edges :" + a.geneNetworkMax.numEdges);
		System.out.println("Max nodes :" + a.geneNetworkMax.numNodes);

		//System.out.println("R edges :" + a.randomGeneNetwork.numEdges);
		//System.out.println("R nodes :" + a.randomGeneNetwork.numNodes);

		//a.truncGeneNetwork(a.randomGeneNetwork.maxWeight());
		//System.err.println("edges :" + a.geneNetworkSum.numEdges);
		//System.err.println("nodes :" + a.geneNetworkSum.numNodes);

		System.out.println("Sorting edges for Sum Network");
		a.geneNetworkSum.sortEdges();

		System.out.println("Sorting edges for Max Network");
		a.geneNetworkMax.sortEdges();

		//System.out.println("Writing edges files...");
		//a.geneNetwork.printEdges("edges.txt", 0.0);

		System.out.println("Writing files...");
		try {
			BufferedWriter out = new BufferedWriter(new FileWriter("network_sum",false));
			for (Edge e : a.geneNetworkSum.edgeList) {
				out.write(e.source + "\t" + e.target + "\t" + e.weight + "\n");
			}
			out.close();

			out = new BufferedWriter(new FileWriter("network_max",false));
			for (Edge e : a.geneNetworkMax.edgeList) {
				out.write(e.source + "\t" + e.target + "\t" + e.weight + "\n");
			}
			out.close();

			System.err.println("Done.");
		}
		catch (Exception e) {
			System.err.println(e);
		}
	} // end analyze


	// for microRNA - read
	public static void read(String file) {

		ModuleNetwork m = new ModuleNetwork(file);
		HashMap hmap = new HashMap<String, String>();

		try {
			BufferedReader br = new BufferedReader (new FileReader (new File("/group/biocomp/projects/segal/mirna/affy_data/all.hugo.map")));
			String line;
			while((line = br.readLine()) != null) {
				String[] data = line.split("\\t");
				//System.out.println(data[1]);
				hmap.put(data[0], data[1]);
			}
		}
		catch (IOException e) {
			e.printStackTrace();
			System.exit(1);
		}


		for (Gene g : m.regulatorSet) {

			if (hmap.get(g.name) != null)
				//System.out.println(hmap.get(g.name));
				g.name = (String) hmap.get(g.name);
		}

		///*
		// draw modules genes + regulation program
		System.out.println("Drawing modules...");
		for (Module mod : m.moduleSet) {
			String filename = "module" + mod.number + ".png";
			mod.draw(filename);
		}
		System.out.println("Done.");
		//*/

		/*
		for (Module mod : m.moduleSet) {
			stat(mod.number, mod.hierarchicalTree, m.numGenes, mod.genes.size());
		}
		 */
	}


	public static void stock_ath (String filename) {

		filename = filename + ".xml.gz";

		String dataDir = "/group/sysbiol/java/test_data/";
		//String genefile = "cc_data";
		String genefile = "cc_data_sd_0.5";
		//String genefile = "cc_data_test100";
		String regfile = "ath_go_plus_cc_regulators.txt";

		double lambda0 = 0.1;
		double mu0 = 0.0;
		double alpha0 = 0.1;
		double beta0 = 0.1;

		double scoregain = 0.0005;
		int numRunsGenes = 1;
//		int numStepsGenes = 2;
//		int burnInGenes = 1;
//		int sampleStepGenes = 2;
		int numStepsGenes = 100;
		int burnInGenes = 50;
		int sampleStepGenes = 100;

		int initNumClust = 6000;

		boolean useBHCscore = true;

		// print parameters in the log file
		System.out.println("######## Parameters ##########");
		System.out.println("data directory: " + dataDir);
		System.out.println("data file: " + genefile);
		System.out.println("regulators: " + regfile);
		System.out.println("Initial nb. clusters: " + initNumClust);
		System.out.println("scoregain: " + scoregain);
		System.out.println("useBHCscore: " + useBHCscore);
		System.out.println("numStepsGenes: " + numStepsGenes);
		System.out.println("burnInGenes: " + burnInGenes);
		System.out.println("sampleStepGenes: " + sampleStepGenes);
		System.out.println("initNumClust: " + initNumClust);
		System.out.println("##############################");


		HashMap<String,String> empty = new HashMap<String,String>();

		// Create ModuleNetwork object
		ModuleNetwork M = new ModuleNetwork(dataDir, genefile, regfile, lambda0, mu0, alpha0, beta0, empty);

		// Gibbs sample different module sets with one tree per module
		M.gibbsSamplerGenes(initNumClust, numRunsGenes, numStepsGenes, burnInGenes, sampleStepGenes, scoregain, useBHCscore);

		System.out.println("Final nb clusters: " + M.moduleSet.size());

		M.writeXML(filename);

	}
	
	public static void mir_learn_reg_for_chris_town_data (String filename) {

		filename = filename + ".xml.gz";

		// Set data directory
		String dataDir = "/group/sysbiol/Chris_Town_data/";

		// complete dataset
		String geneFile =      "affy_log_norm_hv.txt";
		String clusterFile =   "lemone_clustering_tight.txt";
		String mirFile =       "pcr_matrix2";
		String regFile =       "pcr_reg2";

		// Set initial number of clusters
		int numclust = 70;

		// Set normal-gamma prior paramaters
		double lambda0 = 0.1;
		double mu0 =     0.0;
		double alpha0 =  0.1;
		double beta0 =   0.1;

		double scoregain = 0.005;
		boolean useBHCscore = true;

		int numRunsExpts = 1;
		int numStepsExpts = 1100;
		int burnInExpts = 100;
		int sampleStepExpts = 10;

		//int numStepsExpts = 2;
		//int burnInExpts = 1;
		//int sampleStepExpts = 2;

		//int cutLevel = 2;
		//int minSizeExpt = 10;
		//int minSizeExpt = 16;
		//double signiFact = 1;

		double betaReg = 75;
		int numReg = 100;
		//int numReg = 1;

		HashMap<String, String> fake = new HashMap<String, String>();

		// print parameters in the log file
		System.out.println("data file: " + geneFile);
		System.out.println("cluster file: " + clusterFile);
		System.out.println("mir file: " + mirFile);
		System.out.println("regulators: " + regFile);
		System.out.println("Nb clusters: " + numclust);
		System.out.println("scoregain: " + scoregain);
		System.out.println("useBHCscore: " + useBHCscore);
		System.out.println("numRunsExpts: " + numRunsExpts);
		System.out.println("numStepsExpts: " + numStepsExpts);
		System.out.println("burnInExpts: " + burnInExpts);
		System.out.println("sampleStepExpts: " + sampleStepExpts);
		System.out.println("betaReg: " + betaReg);
		System.out.println("numReg: " + numReg);

		// Construct module network
		ModuleNetwork M = new ModuleNetwork(dataDir, geneFile, mirFile, numclust, clusterFile, regFile, lambda0, mu0, alpha0, beta0, fake);

		M.dataType = M.dataType.AFFY;

		// rescale scoregain
		//scoregain = scoregain * M.numGenes;

		M.gibbsSamplerExpts(numRunsExpts, numStepsExpts, burnInExpts, sampleStepExpts, scoregain, useBHCscore);

		//M.makeConsensusTrees(cutLevel, minSizeExpt, signiFact);

		M.assignRegulatorsNoAcyclStoch(betaReg, numReg);

		M.writeMicroXML(filename);

	}
	
	public static void print_reg_score(String dir) {
		
		ModuleNetwork m = new ModuleNetwork(dir, "",false);
		
		try{
            File output = new File("regweight.txt");
            PrintWriter pw = new PrintWriter(output);
            for (Module mod : m.moduleSet){
                if (!mod.genes.isEmpty())
                    for (Gene reg : mod.regulatorWeights.keySet()){
                        String line = reg.name + "\t" + String.format("%d", mod.number+1) + "\t"
                            + String.format("%10.6f", mod.regulatorWeights.get(reg));
                        pw.println(line);
                    }
            }
            pw.close();
        } catch (IOException e){
            System.out.println(e);
        }
	}
	
	public static void prost (String data_dir) {

		String mapfile = "/group/sysbiol/erbon/prostate_cancer/run1/all_genes_map";
		String top_reg = "/group/sysbiol/erbon/prostate_cancer/run1/reg.txt";
		
		
		// load map file for gene names
		HashMap<String, String> map = new HashMap<String, String>();
		if (mapfile != null) {
			System.out.println("Loading gene map file..");
			try {
				BufferedReader input = new BufferedReader(new FileReader(new File(mapfile)));
				String line;
				while((line = input.readLine()) != null) {
					line.trim();
					String tk[] = line.split("\\t");
					map.put(tk[0], tk[1]);
				}
				input.close();
			}
			catch (FileNotFoundException e) {
				System.out.println("Error: file not found");
				e.printStackTrace();
				System.exit(1);
			}
			catch (IOException e) {
				System.out.println("Error while parsing the map file");
				e.printStackTrace();
				System.exit(1);
			}
		}
		
		
		int cut_level = 0;
		ModuleNetwork M = new ModuleNetwork(data_dir,"",false);
		M.setTopRegulators(top_reg);
		M.setModuleMeanSigma();
		M.setRegulatorMeanSigma();
		M.checkExperiments();
		M.changeGeneNames(map);
		// cut trees to a certain level
		if (cut_level > 0) {
			for (Module mod : M.moduleSet) {
				for (TreeNode t : mod.hierarchicalTrees) {
					t.testLevel(cut_level);
				}
			}
		}
		M.setGlobalMeanForFigures(true);
		M.setRegulatorlMeanForFigures(true);
		
		Module mod = M.moduleSet.get(5);
		for (int nb=0;nb < mod.hierarchicalTrees.size(); nb++) {
			TreeNode t = mod.hierarchicalTrees.get(nb);
			mod.hierarchicalTree = t;
			if (mod.hierarchicalTree.nodeStatus.equals("internal")) {
				String file = "module_" + mod.number + "_tree_" + nb;
				System.out.println("Drawing " + file + " ...");
				mod.drawEps(file);
			}
		}
	}
}


