function s = clustercolumns(m, numSteps, burnIn, sampleStep)
% CLUSTERCOLUMNS - Clusters columns in a matrix using GaneSh Gibbs sampling
%   s = clustercolumns(m, numSteps, burnIn, sampleStep)
%     m          : input data, columns in m are clustered using the Gibbs 
%                  sampling clustering algorithm GaneSh
%     numSteps   : number of Gibbs sampler steps
%     burnIn     : number of burn in steps before sampling
%     sampleStep : number of steps between samples
%     s          : output matrix of pairwise clustering frequencies
%
%   Make sure to run (and perhaps modify) "setclasspath.m" first!
  
  import be.ugent.psb.GaneSh.*;
  
  % construct Gibbs sampler object, GaneSh by default clusters rows, so transpose m.
  gibbs = GibbsSampler(m');
  
  % run sampler, output gives cluster membership of each column in each sample
  inclust = gibbs.Run(numSteps, burnIn, sampleStep);
  
  % convert to pairwise frequencies
  % number of samples
  K = size(inclust,2)-1;
  
  % initialize pairwise probability matrix
  N = size(inclust,1);
  s = sparse(N,N);
  
  % compute pairwise matrix of 0/1 for each cluster set and add to s
  for l=2:K+1
    % columns in v are clusters, entry 1 means gene belongs to cluster
    v = sparse(N,max(inclust(:,l)));
    for g=1:N
      v(g,inclust(g,l)) = 1;
    end
    % add pairwise matrix to s
    s = s + v*v';
  end
  
  % make probabilities
  s = s./K;
  
