function [s,data,idx] = makepairwiseprob(file)
% MAKEPAIRWISEPROB - read a file with different partitions and convert to pairwise probabilities
% [s,data,idx] = makepairwiseprob(file)
%    file : name of input file, should have the format:
%           ID name, ID number, cluster 1, cluster 2, ...
%    s    : pairwise probability matrix
%    idx  : cell array with names of genes corresponding to rows/columns
%           in s
  

  % read ID names, numbers and clusters
  str = importdata(file);

  % data matrix : 1st column = gene numbers (0,1,2,...), other colums are
  % cluster numbers (0,1,2,...)
  data = str.data;
  
  % in matlab, it's easier if indices start with 1
  if min(data(:,1))==0
    data = data + ones(size(data));
  end
  
  % number of genes
  N = size(data,1);
  
  % number of cluster sets
  K = size(data,2)-1;
  
  % initialize pairwise probability matrix
  s = sparse(N,N);
  
  % compute pairwise matrix of 0/1 for each cluster set and add to s
  for l=2:K+1
    % columns in v are clusters, entry 1 means gene belongs to cluster
    v = sparse(N,max(data(:,l)));
    for g=1:N
      v(g,data(g,l)) = 1;
    end
    % add pairwise matrix to s
    s = s + v*v';
  end
  
  % make probabilities
  s = s./K;

  % output gene names
  idx = str.textdata;
